# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from ..core.serialization import FieldMetadata
import pydantic
from .order_status import OrderStatus
import datetime as dt
from .order_dispute_last_status import OrderDisputeLastStatus
from .order_price import OrderPrice
from .order_address import OrderAddress
from .order_customer_info import OrderCustomerInfo
from .order_purchased_item import OrderPurchasedItem
from .stripe_details import StripeDetails
from .stripe_card import StripeCard
from .paypal_details import PaypalDetails
from .order_metadata import OrderMetadata
from .order_totals import OrderTotals
from .order_download_files_item import OrderDownloadFilesItem
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Order(UniversalBaseModel):
    order_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="orderId")] = pydantic.Field(
        default=None
    )
    """
    The order ID. Will usually be 6 hex characters, but can also be 9
    hex characters if the site has a very large number of Orders.
    Randomly assigned.
    """

    status: typing.Optional[OrderStatus] = pydantic.Field(default=None)
    """
    The status of the Order
    """

    comment: typing.Optional[str] = pydantic.Field(default=None)
    """
    A comment string for this Order, which is editable by API user (not used by Webflow).
    """

    order_comment: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="orderComment")] = (
        pydantic.Field(default=None)
    )
    """
    A comment that the customer left when making their Order
    """

    accepted_on: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="acceptedOn")] = (
        pydantic.Field(default=None)
    )
    """
    The ISO8601 timestamp that an Order was placed.
    """

    fulfilled_on: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="fulfilledOn")] = (
        pydantic.Field(default=None)
    )
    """
    When an Order is marked as 'fulfilled', this field represents the timestamp of the fulfillment in ISO8601 format. Otherwise, it is null.
    """

    refunded_on: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="refundedOn")] = (
        pydantic.Field(default=None)
    )
    """
    When an Order is marked as 'refunded', this field represents the timestamp of the fulfillment in ISO8601 format. Otherwise, it is null.
    """

    disputed_on: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="disputedOn")] = (
        pydantic.Field(default=None)
    )
    """
    When an Order is marked as 'disputed', this field represents the timestamp of the fulfillment in ISO8601 format. Otherwise, it is null.
    """

    dispute_updated_on: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="disputeUpdatedOn")
    ] = pydantic.Field(default=None)
    """
    If an Order has been disputed by the customer, this key will be set to the ISO8601 timestamp of the last update received. If the Order is not disputed, the key will be null.
    """

    dispute_last_status: typing_extensions.Annotated[
        typing.Optional[OrderDisputeLastStatus], FieldMetadata(alias="disputeLastStatus")
    ] = pydantic.Field(default=None)
    """
    If an order was disputed by the customer, then this key will be set with the [dispute's status](https://stripe.com/docs/api#dispute_object-status).
    """

    customer_paid: typing_extensions.Annotated[typing.Optional[OrderPrice], FieldMetadata(alias="customerPaid")] = (
        pydantic.Field(default=None)
    )
    """
    The total paid by the customer
    """

    net_amount: typing_extensions.Annotated[typing.Optional[OrderPrice], FieldMetadata(alias="netAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The net amount after application fees
    """

    application_fee: typing_extensions.Annotated[typing.Optional[OrderPrice], FieldMetadata(alias="applicationFee")] = (
        pydantic.Field(default=None)
    )
    """
    The application fee assessed by the platform
    """

    all_addresses: typing_extensions.Annotated[
        typing.Optional[typing.List[OrderAddress]], FieldMetadata(alias="allAddresses")
    ] = pydantic.Field(default=None)
    """
    All addresses provided by the customer during the ordering flow.
    """

    shipping_address: typing_extensions.Annotated[
        typing.Optional[OrderAddress], FieldMetadata(alias="shippingAddress")
    ] = pydantic.Field(default=None)
    """
    The shipping address
    """

    billing_address: typing_extensions.Annotated[
        typing.Optional[OrderAddress], FieldMetadata(alias="billingAddress")
    ] = pydantic.Field(default=None)
    """
    The billing address
    """

    shipping_provider: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="shippingProvider")] = (
        pydantic.Field(default=None)
    )
    """
    A string editable by the API user to note the shipping provider used (not used by Webflow).
    """

    shipping_tracking: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="shippingTracking")] = (
        pydantic.Field(default=None)
    )
    """
    A string editable by the API user to note the shipping tracking number for the order (not used by Webflow).
    """

    shipping_tracking_url: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="shippingTrackingURL")
    ] = None
    customer_info: typing_extensions.Annotated[
        typing.Optional[OrderCustomerInfo], FieldMetadata(alias="customerInfo")
    ] = pydantic.Field(default=None)
    """
    An object with the keys `fullName` and `email`.
    """

    purchased_items: typing_extensions.Annotated[
        typing.Optional[typing.List[OrderPurchasedItem]], FieldMetadata(alias="purchasedItems")
    ] = pydantic.Field(default=None)
    """
    An array of all things that the Customer purchased.
    """

    purchased_items_count: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="purchasedItemsCount")
    ] = pydantic.Field(default=None)
    """
    The sum of all 'count' fields in 'purchasedItems'.
    """

    stripe_details: typing_extensions.Annotated[
        typing.Optional[StripeDetails], FieldMetadata(alias="stripeDetails")
    ] = None
    stripe_card: typing_extensions.Annotated[typing.Optional[StripeCard], FieldMetadata(alias="stripeCard")] = None
    paypal_details: typing_extensions.Annotated[
        typing.Optional[PaypalDetails], FieldMetadata(alias="paypalDetails")
    ] = None
    custom_data: typing_extensions.Annotated[
        typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]], FieldMetadata(alias="customData")
    ] = pydantic.Field(default=None)
    """
    An array of additional inputs for custom order data gathering. Each object in the array represents an input with a name, and a textInput, textArea, or checkbox value.
    """

    metadata: typing.Optional[OrderMetadata] = None
    is_customer_deleted: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="isCustomerDeleted")
    ] = pydantic.Field(default=None)
    """
    A boolean indicating whether the customer has been deleted from the site.
    """

    is_shipping_required: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="isShippingRequired")
    ] = pydantic.Field(default=None)
    """
    A boolean indicating whether the order contains one or more purchased items that require shipping.
    """

    has_downloads: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="hasDownloads")] = (
        pydantic.Field(default=None)
    )
    """
    A boolean indicating whether the order contains one or more purchased items that are downloadable.
    """

    payment_processor: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="paymentProcessor")] = (
        pydantic.Field(default=None)
    )
    """
    A string indicating the payment processor used for this order.
    """

    totals: typing.Optional[OrderTotals] = pydantic.Field(default=None)
    """
    An object describing various pricing totals
    """

    download_files: typing_extensions.Annotated[
        typing.Optional[typing.List[OrderDownloadFilesItem]], FieldMetadata(alias="downloadFiles")
    ] = pydantic.Field(default=None)
    """
    An array of downloadable file objects.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
